/* eslint-disable no-magic-numbers */
import { Feature } from 'ol';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useEffect, useMemo } from 'react';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import MapElement from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/MapElement';
import { useSelector } from 'react-redux';
import { bioShapesSelector, lineTypesSelector } from '@/redux/shapes/shapes.selectors';
import { MapInstance } from '@/types/map';
import {
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';
import { modelElementsSelector } from '@/redux/modelElements/modelElements.selector';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { getModelElements } from '@/redux/modelElements/modelElements.thunks';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import CompartmentSquare from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentSquare';
import CompartmentCircle from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentCircle';
import { ModelElement } from '@/types/models';

export const useOlMapReactionsLayer = ({
  mapInstance,
}: {
  mapInstance: MapInstance;
}): VectorLayer<VectorSource<Feature>> => {
  const dispatch = useAppDispatch();
  const modelElements = useSelector(modelElementsSelector);
  const currentModelId = useSelector(currentModelIdSelector);
  useEffect(() => {
    dispatch(getModelElements(currentModelId));
  }, [currentModelId, dispatch]);

  const pointToProjection = usePointToProjection();
  const shapes = useSelector(bioShapesSelector);
  const lineTypes = useSelector(lineTypesSelector);

  const elements: Array<MapElement | CompartmentCircle | CompartmentSquare> = useMemo(() => {
    if (!modelElements || !shapes) return [];

    const validElements: Array<MapElement | CompartmentCircle | CompartmentSquare> = [];
    modelElements.content.forEach((element: ModelElement) => {
      const shape = shapes.find(bioShape => bioShape.sboTerm === element.sboTerm);
      if (shape) {
        validElements.push(
          new MapElement({
            shapes: shape.shapes,
            x: element.x,
            y: element.y,
            nameX: element.nameX,
            nameY: element.nameY,
            nameHeight: element.nameHeight,
            nameWidth: element.nameWidth,
            width: element.width,
            height: element.height,
            zIndex: element.z,
            lineWidth: element.lineWidth,
            lineType: element.borderLineType,
            fontColor: element.fontColor,
            fillColor: element.fillColor,
            borderColor: element.borderColor,
            nameVerticalAlign: element.nameVerticalAlign as VerticalAlign,
            nameHorizontalAlign: element.nameHorizontalAlign as HorizontalAlign,
            homodimer: element.homodimer,
            activity: element.activity,
            text: element.name,
            fontSize: element.fontSize,
            pointToProjection,
            mapInstance,
            modifications: element.modificationResidues,
            lineTypes,
            bioShapes: shapes,
          }),
        );
      } else if (element.sboTerm === 'SBO:0000290') {
        const compartmentProps = {
          x: element.x,
          y: element.y,
          nameX: element.nameX,
          nameY: element.nameY,
          nameHeight: element.nameHeight,
          nameWidth: element.nameWidth,
          width: element.width,
          height: element.height,
          zIndex: element.z,
          innerWidth: element.innerWidth,
          outerWidth: element.outerWidth,
          thickness: element.thickness,
          fontColor: element.fontColor,
          fillColor: element.fillColor,
          borderColor: element.borderColor,
          nameVerticalAlign: element.nameVerticalAlign as VerticalAlign,
          nameHorizontalAlign: element.nameHorizontalAlign as HorizontalAlign,
          text: element.name,
          fontSize: element.fontSize,
          pointToProjection,
          mapInstance,
        };
        if (element.shape === 'OVAL_COMPARTMENT') {
          validElements.push(new CompartmentCircle(compartmentProps));
        } else if (element.shape === 'SQUARE_COMPARTMENT') {
          validElements.push(new CompartmentSquare(compartmentProps));
        }
      }
    });
    return validElements;
  }, [modelElements, shapes, pointToProjection, mapInstance, lineTypes]);

  const features = useMemo(() => {
    return elements.map(element => element.multiPolygonFeature);
  }, [elements]);

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features,
    });
  }, [features]);

  return useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );
};
