/* eslint-disable no-magic-numbers */
import { Style, Text } from 'ol/style';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import getStroke from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/style/getStroke';
import getFill from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/style/getFill';
import Polygon from 'ol/geom/Polygon';
import getMultiPolygon from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/getMultiPolygon';
import { rgbToHex } from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/style/rgbToHex';
import { BioShape, LineType, Modification, Shape } from '@/types/models';
import { MapInstance } from '@/types/map';
import {
  ColorObject,
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';
import {
  BLACK_COLOR,
  WHITE_COLOR,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.constants';
import BaseMultiPolygon from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/BaseMultiPolygon';
import getStyle from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/style/getStyle';

export type MapElementProps = {
  shapes: Array<Shape>;
  x: number;
  y: number;
  width: number;
  height: number;
  zIndex: number;
  fillColor?: ColorObject;
  borderColor?: ColorObject;
  fontColor?: ColorObject;
  lineWidth?: number;
  lineType?: string;
  text?: string;
  fontSize?: number;
  nameX: number;
  nameY: number;
  nameHeight: number;
  nameWidth: number;
  nameVerticalAlign?: VerticalAlign;
  nameHorizontalAlign?: HorizontalAlign;
  homodimer?: number;
  activity?: boolean;
  pointToProjection: UsePointToProjectionResult;
  mapInstance: MapInstance;
  bioShapes?: Array<BioShape>;
  lineTypes?: Array<LineType>;
  modifications?: Array<Modification>;
};

export default class MapElement extends BaseMultiPolygon {
  shapes: Array<Shape>;

  lineWidth: number;

  lineType: string | undefined;

  bioShapes: Array<BioShape>;

  lineTypes: Array<LineType>;

  homodimer: number;

  activity: boolean | undefined;

  modifications: Array<Modification>;

  lineDash: Array<number> = [];

  constructor({
    shapes,
    x,
    y,
    width,
    height,
    zIndex,
    fillColor = WHITE_COLOR,
    borderColor = BLACK_COLOR,
    fontColor = BLACK_COLOR,
    lineWidth = 1,
    lineType,
    text = '',
    fontSize = 12,
    nameX,
    nameY,
    nameHeight,
    nameWidth,
    nameVerticalAlign = 'MIDDLE',
    nameHorizontalAlign = 'CENTER',
    homodimer = 1,
    activity,
    pointToProjection,
    mapInstance,
    bioShapes = [],
    lineTypes = [],
    modifications = [],
  }: MapElementProps) {
    super({
      x,
      y,
      width,
      height,
      zIndex,
      text,
      fontSize,
      nameX,
      nameY,
      nameWidth,
      nameHeight,
      fontColor,
      nameVerticalAlign,
      nameHorizontalAlign,
      fillColor,
      borderColor,
      pointToProjection,
    });
    this.shapes = shapes;
    this.lineWidth = lineWidth;
    this.lineType = lineType;
    this.homodimer = homodimer;
    this.activity = activity;
    this.bioShapes = bioShapes;
    this.lineTypes = lineTypes;
    this.modifications = modifications;
    this.createPolygons();
    this.drawText();
    this.drawMultiPolygonFeature(mapInstance);
  }

  protected createPolygons(): void {
    let multiPolygonModifications: Array<Polygon> = [];
    this.modifications.forEach(modification => {
      if (modification.state === null) {
        return;
      }

      const shape = this.bioShapes.find(bioShape => bioShape.sboTerm === modification.sboTerm);
      if (!shape) {
        return;
      }
      multiPolygonModifications = getMultiPolygon({
        x: modification.x,
        y: modification.y,
        width: modification.width,
        height: modification.height,
        shapes: shape.shapes,
        pointToProjection: this.pointToProjection,
        mirror: modification.direction && modification.direction === 'RIGHT',
      });
      this.polygons.push(...multiPolygonModifications);
      multiPolygonModifications.forEach(polygon => {
        const modificationStyle = new Style({
          geometry: polygon,
          stroke: getStroke({ color: rgbToHex(modification.borderColor) }),
          fill: getFill({ color: rgbToHex(modification.fillColor) }),
          zIndex: modification.z,
        });
        const modificationText = modification.stateAbbreviation
          ? modification.stateAbbreviation
          : modification.name;
        if (modificationText) {
          modificationStyle.setText(
            new Text({
              text: modificationText,
              font: `${modification.fontSize}px Arial`,
              textAlign: 'center',
              textBaseline: 'middle',
              fill: getFill({ color: '#000' }),
              overflow: true,
            }),
          );
          this.polygonsTexts.push(modification.name);
        }
        this.styles.push(modificationStyle);
      });
    });

    if (this.lineType) {
      const lineTypeFound = this.lineTypes.find(type => type.name === this.lineType);
      if (lineTypeFound) {
        this.lineDash = lineTypeFound.pattern;
      }
    }

    const homodimerOffset = (this.homodimer - 1) * 6;
    for (let i = 0; i < this.homodimer; i += 1) {
      const homodimerShift = (this.homodimer - i - 1) * 6;
      if (this.activity) {
        this.drawActiveBorder(homodimerShift, homodimerOffset);
      }
      this.drawElementPolygon(homodimerShift, homodimerOffset);
    }
  }

  drawActiveBorder(homodimerShift: number, homodimerOffset: number): void {
    const activityBorderElement = getMultiPolygon({
      x: this.x + homodimerShift - 5,
      y: this.y + homodimerShift - 5,
      width: this.width - homodimerOffset + 10,
      height: this.height - homodimerOffset + 10,
      shapes: this.shapes,
      pointToProjection: this.pointToProjection,
    });
    activityBorderElement.forEach(polygon => {
      this.styles.push(
        getStyle({
          geometry: polygon,
          fillColor: { rgb: 0, alpha: 0 },
          lineDash: [3, 5],
          zIndex: this.zIndex,
        }),
      );
    });
    this.polygons.push(...activityBorderElement);
  }

  drawElementPolygon(homodimerShift: number, homodimerOffset: number): void {
    const elementPolygon = getMultiPolygon({
      x: this.x + homodimerShift,
      y: this.y + homodimerShift,
      width: this.width - homodimerOffset,
      height: this.height - homodimerOffset,
      shapes: this.shapes,
      pointToProjection: this.pointToProjection,
    });
    elementPolygon.forEach(polygon => {
      this.styles.push(
        getStyle({
          geometry: polygon,
          borderColor: this.borderColor,
          fillColor: this.fillColor,
          lineWidth: this.lineWidth,
          lineDash: this.lineDash,
          zIndex: this.zIndex,
        }),
      );
    });
    this.polygons.push(...elementPolygon);
  }
}
